var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Functions_native, _Functions_app;
import { HttpsErrorCode } from "./common";
import { deserialize, firebase, FirebaseApp, serialize } from "@nativescript/firebase-core";
let defaultFunctions;
const fb = firebase();
Object.defineProperty(fb, 'functions', {
    value: (app) => {
        if (!app) {
            if (!defaultFunctions) {
                defaultFunctions = new Functions();
            }
            return defaultFunctions;
        }
        return new Functions(app);
    },
    writable: false,
});
/**
  Firebase Functions Region - Region for which to run HttpsCallable method
  Set parameter using firebase().app().functions(regionOrCustomDomain: string)
  @link https://firebase.google.com/docs/reference/node/firebase.app.App
  @link https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions
  @note If not set, default region is used ('us-central1')
  */
let defaultRegionOrCustomDomain;
/**
  Add 'functions' method to FirebaseApp class
  @param regionOrCustomDomain(string)(Optional): Name of the Region or Custom Domain for which to Functions results
  @return Functions
  @see FirebaseFunctions
  @link https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions
  @see Supported Regions
  @see https://firebase.google.com/docs/functions/locations
  */
const fbApp = FirebaseApp;
Object.defineProperty(fbApp.prototype, 'functions', {
    value: (regionOrCustomDomain) => {
        defaultRegionOrCustomDomain = regionOrCustomDomain;
        if (!defaultFunctions) {
            defaultFunctions = new Functions();
        }
        return defaultFunctions;
    },
    writable: false,
});
function errorToCode(error) {
    let code = HttpsErrorCode.UNKNOWN;
    switch (error.code) {
        case 0 /* OK */:
            code = HttpsErrorCode.OK;
            break;
        case 1 /* Cancelled */:
            code = HttpsErrorCode.CANCELLED;
            break;
        case 2 /* Unknown */:
            code = HttpsErrorCode.UNKNOWN;
            break;
        case 3 /* InvalidArgument */:
            code = HttpsErrorCode.INVALID_ARGUMENT;
            break;
        case 4 /* DeadlineExceeded */:
            code = HttpsErrorCode.DEADLINE_EXCEEDED;
            break;
        case 5 /* NotFound */:
            code = HttpsErrorCode.NOT_FOUND;
            break;
        case 6 /* AlreadyExists */:
            code = HttpsErrorCode.ALREADY_EXISTS;
            break;
        case 7 /* PermissionDenied */:
            code = HttpsErrorCode.PERMISSION_DENIED;
            break;
        case 8 /* ResourceExhausted */:
            code = HttpsErrorCode.RESOURCE_EXHAUSTED;
            break;
        case 9 /* FailedPrecondition */:
            code = HttpsErrorCode.FAILED_PRECONDITION;
            break;
        case 10 /* Aborted */:
            code = HttpsErrorCode.ABORTED;
            break;
        case 11 /* OutOfRange */:
            code = HttpsErrorCode.OUT_OF_RANGE;
            break;
        case 12 /* Unimplemented */:
            code = HttpsErrorCode.UNIMPLEMENTED;
            break;
        case 13 /* Internal */:
            code = HttpsErrorCode.INTERNAL;
            break;
        case 14 /* Unavailable */:
            code = HttpsErrorCode.UNAVAILABLE;
            break;
        case 15 /* DataLoss */:
            code = HttpsErrorCode.DATA_LOSS;
            break;
        case 16 /* Unauthenticated */:
            code = HttpsErrorCode.UNAUTHENTICATED;
            break;
        default:
            break;
    }
    return code;
}
export class HttpsError extends Error {
    constructor(code, message, details = null, native = null) {
        super(message);
        this.code = code;
        this.details = details;
        this.native = native;
    }
}
function toHttpsError(error) {
    let details = null;
    if (error.domain == FIRFunctionsErrorDomain) {
        details = error.userInfo[FIRFunctionsErrorDetailsKey];
    }
    return new HttpsError(errorToCode(error), error.localizedDescription, details, error);
}
export class Functions {
    constructor(app) {
        _Functions_native.set(this, void 0);
        _Functions_app.set(this, void 0);
        if (app === null || app === void 0 ? void 0 : app.native) {
            if (defaultRegionOrCustomDomain) {
                __classPrivateFieldSet(this, _Functions_native, isRegion(defaultRegionOrCustomDomain) // Check whether a Region has been set
                    ? FIRFunctions.functionsForAppRegion(app.native, defaultRegionOrCustomDomain) // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions#+functionsforapp:region:
                    : isCustomDomain(defaultRegionOrCustomDomain) // Check whether using a Custom Domain has been set
                        ? FIRFunctions.functionsForAppCustomDomain(app.native, defaultRegionOrCustomDomain) // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions#+functionsforapp:customdomain:
                        : FIRFunctions.functionsForApp(app.native), "f"); // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions#+functionsforapp:
            }
            else {
                __classPrivateFieldSet(this, _Functions_native, FIRFunctions.functionsForApp(app.native), "f"); // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions#+functionsforapp:
            }
        }
        else {
            if (defaultFunctions) {
                return defaultFunctions;
            }
            defaultFunctions = this;
            if (defaultRegionOrCustomDomain) {
                __classPrivateFieldSet(this, _Functions_native, isRegion(defaultRegionOrCustomDomain) // Check whether a Region has been set
                    ? FIRFunctions.functionsForRegion(defaultRegionOrCustomDomain) // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions#+functionsforregion:
                    : isCustomDomain(defaultRegionOrCustomDomain) // Check whether using a Custom Domain has been set
                        ? FIRFunctions.functionsForCustomDomain(defaultRegionOrCustomDomain) // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions#+functionsforcustomdomain:
                        : FIRFunctions.functions(), "f"); // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions#+functions
            }
            else {
                __classPrivateFieldSet(this, _Functions_native, FIRFunctions.functions(), "f"); // @see https://firebase.google.com/docs/reference/ios/firebasefunctions/api/reference/Classes/FIRFunctions#+functions
            }
        }
    }
    httpsCallable(name, options) {
        const callable = this.native.HTTPSCallableWithName(name);
        if (typeof (options === null || options === void 0 ? void 0 : options.timeout) === 'number') {
            callable.timeoutInterval = options.timeout;
        }
        return (data) => {
            return new Promise((resolve, reject) => {
                if (data) {
                    callable.callWithObjectCompletion(serialize(data), (result, error) => {
                        if (error) {
                            reject(toHttpsError(error));
                        }
                        else {
                            resolve(deserialize(result.data));
                        }
                    });
                }
                else {
                    callable.callWithCompletion((result, error) => {
                        if (error) {
                            reject(toHttpsError(error));
                        }
                        else {
                            resolve(deserialize(result.data));
                        }
                    });
                }
            });
        };
    }
    useEmulator(host, port) {
        this.native.useEmulatorWithHostPort(host, port);
    }
    get native() {
        return __classPrivateFieldGet(this, _Functions_native, "f");
    }
    get ios() {
        return this.native;
    }
    useFunctionsEmulatorOrigin(origin) {
        this.native.useFunctionsEmulatorOrigin(origin);
    }
    get app() {
        if (!__classPrivateFieldGet(this, _Functions_app, "f")) {
            // @ts-ignore
            __classPrivateFieldSet(this, _Functions_app, FirebaseApp.fromNative(this.native.app), "f");
        }
        return __classPrivateFieldGet(this, _Functions_app, "f");
    }
}
_Functions_native = new WeakMap(), _Functions_app = new WeakMap();
/**
  Check whether a regionOrCustomDomain string is a Region for the http trigger, such as “us-central1”.
  @param regionOrCustomDomain(string): Text to parse
  @return boolean: TRUE if a Region; FALSE if not
  */
function isRegion(regionOrCustomDomain) {
    const elems = regionOrCustomDomain.split('.');
    return elems.length === 1 ? true : false;
}
/**
  Check whether a regionOrCustomDomain string is a Custom Domain for the http trigger, such as “https://mydomain.com”
  @param regionOrCustomDomain(string): Text to parse
  @return boolean: TRUE if a CustomDomain; FALSE if not
  */
function isCustomDomain(regionOrCustomDomain) {
    return !isRegion(regionOrCustomDomain);
}
//# sourceMappingURL=index.ios.js.map